#ifndef __lonusb_h__
#define __lonusb_h__

//#define DEBUG  // for dev_dbg()


#include <linux/firmware.h>
#include <linux/kernel.h>

#include <linux/errno.h>
#include <linux/poll.h>
#include <linux/slab.h>
#include <linux/module.h>
#include <linux/completion.h>

#include <linux/usb.h>
#include <linux/version.h>

#include <linux/proc_fs.h>
#include <linux/seq_file.h>

//#include <sys/types.h>
//#include <sys/stat.h>
//#include <unistd.h>

#include "sysdep.h"
#include "GesytecIDs.h"
#include "dynbuf.h"


// use REUSE_URB, if underlaying USB-stack makes some problems
#define REUSE_URB 01

#ifdef CONFIG_PROC_FS
  // enable or disable PROC_FS, if desired
  #define USE_PROC 01   // 0 or 1
#else
  #define USE_PROC 0    // ever 0
#endif


/* Version Information */
#define DRIVER_VERSION	"V4.05"
#define DRIVER_AUTHOR	"Volker Schober, vschober@gesytec.de"
#define DRIVER_DESC	"Easylon LONUSB Driver"
#define DRIVER_MOD_NAME	"lonusb"  // module name
#define DRIVER_DEV_NAME	"lonusb"  // /dev/lonusb
#define PROC_DIR_NAME	"easylon" // /proc/easylon
#define PROC_DEV_NAME	PROC_DIR_NAME "/" DRIVER_DEV_NAME  // /proc/easylon/lonusb


#define IOCTL_VERSION   0x43504C00


/* Get a minor range for your devices from the usb maintainer */
#define LONUSB_MINOR_BASE     0

#define USB_READBUFFER_SIZE     (256+64)    // 64*x
#define USB_WRITEBUFFER_SIZE    256         // 64*x


typedef struct _lonusb_dev_stats
{
	char	ProcDevName[sizeof(PROC_DEV_NAME)+10];

    __u64	open;
	__u64	close;

	__u64	usb_in;
    __u64	read_in;
	__u64	read_ok;
	__u64	read_error;

    __u64	write_in;
	__u64	write_ok;
	__u64	write_error;

	__u64	poll;
	__u64	poll_read;
	__u64	poll_write;
	__u64	poll_error;
	__u64	poll_pollin;
	__u64	poll_pollout;
} lonusb_dev_stats;


/* Structure to hold all of our device specific stuff */
typedef struct _DEVICE_EXTENSION
{
    struct device           dev;        // our device
	struct usb_device	   *usbdev;	    // save off the usb device pointer
	struct usb_interface   *interface;	// the interface for this device
	unsigned char		    minor;	    // the starting minor number for this device

    unsigned char          *intr_in_buff;           // the buffer to receive data
    int                     bulk_in_size;           // the size of the receive buffer
    struct urb             *read_urb;               // the urb used to read data
	__u8			        intr_in_ep_addr;        // the address of the bulk in endpoint
    __u8                    bInterval;              // the polling interval of the bulk in endpoint
    QUEUE                   ReadBufferQueue;        // Data Queue

	wait_queue_head_t	    rd_wait;
	wait_queue_head_t	    wr_wait;
	unsigned char		    intdata[USB_READBUFFER_SIZE];
	unsigned int		    RemainDataSize;	// save incomplete Lon-Telegramm

	unsigned char          *bulk_out_buffer;	/* the buffer to send data */
	int			            bulk_out_size;	/* the size of the send buffer */
#if REUSE_URB
    struct urb             *write_urb;              /* the urb used to send data */
#else
	atomic_t		        write_count;
#endif
	__u8			        bulk_out_endpointAddr;	/* the address of the bulk out endpoint */

	int			            open_count; /* number of times this port has been opened */
	struct mutex		    mutex; /* locks this structure */
	bool			        remove_pending;	/* If set then about to exit */

    lonusb_dev_stats        DevStats; 
} DEVICE_EXTENSION, *PDEVICE_EXTENSION;


static struct usb_driver lonusb_driver;

// local function prototypes
static ssize_t lonusb_read(struct file *file, char __user *buffer, size_t count, loff_t *ppos);
static ssize_t lonusb_write(struct file *file, const char __user *buffer, size_t count, loff_t *ppos);
static unsigned lonusb_poll(struct file *file, struct poll_table_struct *wait);
static long lonusb_ioctl(struct file *file, unsigned int cmd, unsigned long arg);
static int lonusb_open(struct inode *inode, struct file *file);
static int lonusb_release(struct inode *inode, struct file *file);

static int lonusb_probe(struct usb_interface *interface, const struct usb_device_id *id);
static void lonusb_disconnect(struct usb_interface *interface);

static void lonusb_read_int_callback(struct urb *urb);
static void lonusb_delete(PDEVICE_EXTENSION dev);

static void lonusb_debug_data(const char *function, int size, const unsigned char *data);

#endif
