#if USE_PROC

typedef struct _lonusb_proc_stats
{
    __u64	probe_cnt;
    __u64	disconnect_cnt;
} lonusb_proc_stats;


/****************** backport some functions for proc_fs ***********************/
  #if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
static void backport_proc_subdir_remove(struct proc_dir_entry *dir)
{
    struct proc_dir_entry *pe, *tmp;
    pe = dir->subdir;
    while (pe) 
    {
        tmp = pe->next;
        backport_proc_subdir_remove(pe);
        //printk(KERN_DEBUG "remove_proc_entry1(%s)\n", pe->name);
        remove_proc_entry(pe->name, dir);
        pe = tmp;
    }
};

int backport_remove_proc_subtree(const char *name, struct proc_dir_entry *parent)
{
    struct proc_dir_entry *pe, *tmp;
    //printk(KERN_DEBUG "remove_proc_subtree(%s, %p):\n", name, parent);
    
    if (!parent)
        goto out;
    
    pe = parent->subdir;
    while (pe) 
    {
        tmp = pe->next;
        backport_proc_subdir_remove(pe);
        //printk(KERN_DEBUG "remove_proc_entry2(%s)\n", pe->name);
        remove_proc_entry(pe->name, parent);
        pe = tmp;
    }
    
out:
    //printk(KERN_DEBUG "remove_proc_entry3(%s)\n", name);
    remove_proc_entry(name, NULL);
    
    return 0;
}

static inline void *PDE_DATA(const struct inode *inode)
{
    return PROC_I(inode)->pde->data;
}
  #endif
/******************************************************************************/


lonusb_proc_stats ProcStats;
int HasEasylonProcDir = false;


static struct proc_dir_entry* easylon_proc_dir;  // pointer to easylon_proc_dir

int CreateEasylonProcDir(void)
{
    struct file *srcf = filp_open("/proc/" PROC_DIR_NAME, O_DIRECTORY, 0);
    if (IS_ERR(srcf)) 
    {
        easylon_proc_dir = proc_mkdir(PROC_DIR_NAME, NULL);
        if (easylon_proc_dir)
            HasEasylonProcDir = true;
    }
    else
    {
        HasEasylonProcDir = true;
        filp_close(srcf, NULL);
    }

    return HasEasylonProcDir;
}

void RemoveEasylonProcDir(void)
{
#if 0
	if (easylon_proc_dir)
    {
  #if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
		backport_remove_proc_subtree(PROC_DIR_NAME, easylon_proc_dir);
  #else
		remove_proc_subtree(PROC_DIR_NAME, NULL);
  #endif
        easylon_proc_dir = NULL;
    }
#endif
}


static int lonusb_proc_dev_show(struct seq_file *m, void *v) 
{
    PDEVICE_EXTENSION dev = m->private;
    
    if (dev)
    {
        seq_printf(m, "device name:  %s\n",   	&dev->DevStats.ProcDevName[sizeof(PROC_DIR_NAME)]);
//        seq_printf(m, "minor:        %d\n",  	dev->minor);
//        seq_printf(m, "minor_base:   %d\n",  	lonusb_class.minor_base);
        
        seq_printf(m, "open:         %llu\n",   dev->DevStats.open);
        seq_printf(m, "close:        %llu\n",   dev->DevStats.close);
        
        seq_printf(m, "usb_in:       %llu\n", 	dev->DevStats.usb_in);
        seq_printf(m, "read_in:      %llu\n",   dev->DevStats.read_in);
        seq_printf(m, "read_ok:      %llu\n",   dev->DevStats.read_ok);
        seq_printf(m, "read_error:   %llu\n", 	dev->DevStats.read_error);
        seq_printf(m, "queuecnt:     %d\n",  	qcount(&dev->ReadBufferQueue));
        seq_printf(m, "write_in:     %llu\n",   dev->DevStats.write_in);
        seq_printf(m, "write_ok:     %llu\n",   dev->DevStats.write_ok);
        seq_printf(m, "write_error:  %llu\n", 	dev->DevStats.write_error);
        seq_printf(m, "poll:         %llu\n",   dev->DevStats.poll);
        seq_printf(m, "poll_read:    %llu\n",   dev->DevStats.poll_read);
        seq_printf(m, "poll_write:   %llu\n",   dev->DevStats.poll_write);
        seq_printf(m, "poll_error:   %llu\n",   dev->DevStats.poll_error);
        seq_printf(m, "poll_pollin:  %llu\n",   dev->DevStats.poll_pollin);
        seq_printf(m, "poll_pollout: %llu\n",   dev->DevStats.poll_pollout);
    }
    else
        seq_printf(m, "dev=(null)\n");

    return 0;
}

static int lonusb_proc_dev_open(struct inode *inode, struct  file *file) 
{
    return single_open(file, lonusb_proc_dev_show, PDE_DATA(inode));
}

static const struct file_operations lonusb_proc_fops_dev = 
{
    .owner = THIS_MODULE,
    .open = lonusb_proc_dev_open,
    .read = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
};


static int lonusb_proc_all_show(struct seq_file *m, void *v) 
{
    seq_printf(m, "driver name:  %s.ko\n",  DRIVER_MOD_NAME);
    seq_printf(m, "version:      %s\n",     DRIVER_VERSION);
    seq_printf(m, "build date:   %s %s\n",  __DATE__ , __TIME__);
    seq_printf(m, "probe:        %llu\n",   ProcStats.probe_cnt);
    seq_printf(m, "disconnect:   %llu\n",   ProcStats.disconnect_cnt);

    return 0;
}

static int lonusb_proc_all_open(struct inode *inode, struct  file *file) 
{
    return single_open(file, lonusb_proc_all_show, NULL);
}

static const struct file_operations lonusb_proc_fops_all = 
{
    .owner = THIS_MODULE,
    .open = lonusb_proc_all_open,
    .read = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
};

#endif
