#ifndef _SYSDEP_
#define _SYSDEP_

#ifdef __cplusplus
extern "C" {
#endif

//===================================================================
// some Windows types
//===================================================================

#ifndef CONST
#define CONST               const
#endif

// Basics
#ifndef VOID
#define VOID void
typedef void *PVOID;

typedef int8_t CHAR;
typedef int16_t SHORT;
typedef int32_t LONG, INT;
#endif


// Pointer to Basics
typedef CHAR *PCHAR;
typedef CHAR *LPSTR, *PSTR;
typedef CONST CHAR *LPCSTR, *PCSTR;
typedef SHORT *PSHORT;  // winnt
typedef LONG *PLONG;    // winnt

// Unsigned Basics
typedef u_int8_t BYTE;
typedef u_int8_t UCHAR;
typedef u_int16_t USHORT, WORD, *LPWORD;
typedef u_int32_t ULONG, UINT, DWORD, *LPDWORD;

// Pointer to Unsigned Basics
typedef BYTE *PBYTE;
typedef UCHAR *PUCHAR;
typedef USHORT *PUSHORT;
typedef ULONG *PULONG;

typedef UCHAR BOOLEAN;           // winnt
typedef BOOLEAN *PBOOLEAN;       // winnt
#ifndef FALSE
  #define FALSE   0
#endif
#ifndef TRUE
  #define TRUE    1
#endif


//typedef LONG NTSTATUS, *PNTSTATUS;
typedef int32_t NTSTATUS, *PNTSTATUS;

#define _T(A) A
#define _t(A) A

#ifndef min
  //#warning defining min in sysdep.h
  #define min(a,b) ((a) > (b) ? (b) : (a))
  #define MIN min
#else
  //#warning min has been defined outside sysdep.h
  #define MIN(a,b) ((a) > (b) ? (b) : (a))
#endif

#define LOWORD(l)           ((WORD)((DWORD)(l) & 0xffff))
#define HIWORD(l)           ((WORD)((DWORD)(l) >> 16))
#define LOBYTE(w)           ((BYTE)((DWORD)(w) & 0xff))
#define HIBYTE(w)           ((BYTE)((DWORD)(w) >> 8))

#ifndef DECLARE_MUTEX
  #define DECLARE_MUTEX DEFINE_SEMAPHORE
  #define init_MUTEX(m) sema_init(m, 1)
#endif

#ifndef FIELD_OFFSET
  // Calculate the byte offset of a field in a structure of type type.
  #define FIELD_OFFSET(type, field)    ((uintptr_t)&(((type *)0)->field))
#endif



//===================================================================
// Compatibility with older Linux versions
//===================================================================

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
// Kernel 2.4

#define irqreturn_t void
#define IRQ_HANDLED
#define IRQ_NONE

#define iminor(inode) MINOR(inode->i_rdev)
#define imajor(inode) MAJOR(inode->i_rdev)
#define pci_name(pdev) (pdev->slot_name)
#define pm_message_t u32

#define USB_SUBMIT_URB(urb, mem_flags) usb_submit_urb(urb)

#else
// Kernel 2.6

#define USB_SUBMIT_URB(urb, mem_flags) usb_submit_urb(urb, mem_flags)


/* Basic class macros */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,15)
  typedef struct class class_t;
  #define CLASS_DEVICE_CREATE(cls, devt, device, fmt, arg...) class_device_create(cls, NULL, devt, device, fmt, ## arg)
#else /* LINUX 2.6.0 - 2.6.14 */
  #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13) /* LINUX 2.6.13 - 2.6.14 */
    typedef struct class class_t;
    #define CLASS_DEVICE_CREATE class_device_create
  #else /* LINUX 2.6.0 - 2.6.12, class_simple */
    typedef struct class_simple class_t;
    #define CLASS_DEVICE_CREATE class_simple_device_add
    #define class_create class_simple_create
    #define class_destroy class_simple_destroy
    #define class_device_destroy(a, b) class_simple_device_remove(b)
  #endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13) */
#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,15) */

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
  #define DEVICE_CREATE(cls, parent, devt, drvdata, fmt, arg...) device_create(cls, parent, devt, drvdata, fmt, ## arg)
#else
  #define DEVICE_CREATE(cls, parent, devt, drvdata, fmt, arg...) device_create(cls, parent, devt, fmt, ## arg)
#endif

// struct tty_ldisc was renamed to tty_ldisc_ops in 2.6.27
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,27)
  #define tty_ldisc_ops tty_ldisc
#endif

// tty_operations were moved to tty->ops in 2.6.26
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
  #define compat_tty_write(tty) tty->driver->write
#else
  #define compat_tty_write(tty) tty->ops->write
#endif


#endif


#ifdef __cplusplus
	}
#endif


#endif // _SYSDEP_
